@@ -8,18 +8,19 @@ require Exporter;
 @ISA = qw(Exporter);
 
 @EXPORT_OK = qw(
- crc8 crcccitt crc16 crc32 crc64 crc
+ crc8 crcccitt crc16 crcopenpgparmor crc32 crc64 crc
  crc_hex crc_base64
  crcccitt_hex crcccitt_base64
  crc8_hex crc8_base64
  crc16_hex crc16_base64
+ crcopenpgparmor_hex crcopenpgparmor_base64
  crc32_hex crc32_base64
  crc64_hex crc64_base64
 );
 
-$VERSION    = '0.17';
+$VERSION    = '0.19';
 $XS_VERSION = $VERSION;
-$VERSION    = eval $VERSION;
+#$VERSION    = eval $VERSION;
 
 eval {
   # PERL_DL_NONLAZY must be false, or any errors in loading will just
@@ -68,13 +69,13 @@ sub _tabinit($$$) {
     $r = $i if $ref;
     for (my $j=0; $j<8; $j++) {
       if ($ref) {
-	$r = ($r>>1)^($r&1&&$poly)
+        $r = ($r>>1)^($r&1&&$poly)
       } else {
-	if ($r&(1<<($width-1))) {
-	  $r = ($r<<1)^$poly
-	} else {
-	  $r = ($r<<1)
-	}
+        if ($r&(1<<($width-1))) {
+          $r = ($r<<1)^$poly
+        } else {
+          $r = ($r<<1)
+        }
       }
     }
     my $x=$r&2**$width-1;
@@ -126,6 +127,7 @@ ENOXS
   crc8 => [8,0,0,0,0x07,0,0],
   crcccitt => [16,0xffff,0,0,0x1021,0,0],
   crc16 => [16,0,0,1,0x8005,1,0],
+  crcopenpgparmor => [24,0xB704CE,0,0,0x864CFB,0,0],
   crc32 => [32,0xffffffff,0xffffffff,1,0x04C11DB7,1,0],
 );
 
@@ -159,7 +161,7 @@ sub reset {
     $self->{refin} = $typeparams->[5],
     $self->{cont} = $typeparams->[6],
   }
-  $self->{_tab} = _tabinit($self->{width}, $self->{poly}, $self->{refin});
+  $self->{_tab} = defined($self->{width})?_tabinit($self->{width}, $self->{poly}, $self->{refin}):undef;
   $self->{_data} = undef;
   $self
 }
@@ -167,15 +169,19 @@ sub reset {
 #########################################
 # Private output converter functions:
 sub _encode_hex { sprintf "%x", $_[0] }
+
 sub _encode_base64 {
-	my $res;
-	while ($_[0] =~ /(.{1,45})/gs) {
-		$res .= substr pack('u', $1), 1;
-		chop $res;
-	}
-	$res =~ tr|` -_|AA-Za-z0-9+/|;#`
-	chop $res; chop $res;
-	$res
+  my ($res, $padding, $in) = ("", undef, $_[0]);
+  $in = pack("H*", sprintf("%x",$in));
+  while ($in =~ /(.{1,45})/gs) {
+	  $res .= substr pack('u', $1), 1;
+	  chop $res;
+  }
+  $res =~ tr|` -_|AA-Za-z0-9+/|;
+  $padding = (3 - length($in) % 3) % 3;
+  $res =~ s#.{$padding}$#'=' x $padding#e if $padding;
+  $res =~ s#(.{1,76})#$1\n#g;
+  $res
 }
 
 #########################################
@@ -217,8 +223,13 @@ sub digest {
   my $crc;
   if (!$self->{_crc}) {
     my $init = $self->{init};
-    $crc =_crc($self->{_data},$self->{width},$init,$self->{xorout},
+    if (defined($self->{type}) && $self->{type} eq 'crc64' ||
+        defined($self->{width}) && $self->{width} eq 64) {
+      $crc = _crc64($self->{_data});
+    } else {
+      $crc =_crc($self->{_data},$self->{width},$init,$self->{xorout},
 	 $self->{refin},$self->{refout},$self->{cont},$self->{_tab});
+    }
   } else {
     $crc = $self->{_crc};
     $self->{_crc} = undef;
@@ -237,6 +248,7 @@ sub b64digest {
 
 sub clone {
   my $self = shift;
+  use Data::Dumper; print Dumper $self;
   my $clone = { 
     type => $self->{type},
     width => $self->{width},
@@ -245,8 +257,11 @@ sub clone {
     poly => $self->{poly},
     refin => $self->{refin},
     refout => $self->{refout},
-    _data => $self->{_data}
+    _data => $self->{_data},
+    cont => $self->{cont},
+    _tab => $self->{_tab}
   };
+  use Data::Dumper; print Dumper $clone;
   bless $clone, ref $self || $self;
 }
 
@@ -258,32 +273,47 @@ sub crc {
   _crc($message,$width,$init,$xorout,$refin,$refout,$cont,_tabinit($width,$poly,$refin));
 }
 
+sub _cont {
+  my ($message,$init,@parameters) = @_;
+  if (defined $init) {
+    $parameters[1] = $init;
+    $parameters[6] = 1;
+  }
+  crc($message,@parameters);
+}
+
 # CRC8
 # poly: 07, width: 8, init: 00, revin: no, revout: no, xorout: no
 
-sub crc8 { crc($_[0],@{$_typedef{crc8}}) }
+sub crc8 { _cont($_[0],$_[1],@{$_typedef{crc8}}) }
 
 # CRC-CCITT standard
 # poly: 1021, width: 16, init: ffff, refin: no, refout: no, xorout: no
 
-sub crcccitt { crc($_[0],@{$_typedef{crcccitt}}) }
+sub crcccitt { _cont($_[0],$_[1],@{$_typedef{crcccitt}}) }
 
 # CRC16
 # poly: 8005, width: 16, init: 0000, revin: yes, revout: yes, xorout: no
 
-sub crc16 { crc($_[0],@{$_typedef{crc16}}) }
+sub crc16 { _cont($_[0],$_[1],@{$_typedef{crc16}}) }
+
+# CRC-24 for OpenPGP ASCII Armor checksum
+# https://tools.ietf.org/html/rfc4880#section-6
+# poly: 0x864CFB, width: 24, init: 0xB704CE, refin: no, refout: no, xorout: no
+
+sub crcopenpgparmor { crc($_[0],@{$_typedef{crcopenpgparmor}}) }
 
 # CRC32
 # poly: 04C11DB7, width: 32, init: FFFFFFFF, revin: yes, revout: yes,
 # xorout: FFFFFFFF
 # equivalent to: cksum -o3
 
-sub crc32 { crc($_[0],@{$_typedef{crc32}}) }
+sub crc32 { _cont($_[0],$_[1],@{$_typedef{crc32}}) }
 
 # CRC64
 # special XS implementation (_crc64)
 
-sub crc64 { _crc64($_[0]) }
+sub crc64 { _crc64($_[0],defined($_[1])?$_[1]:0) }
 
 sub crc_hex { _encode_hex &crc }
 
@@ -301,6 +331,10 @@ sub crc16_hex { _encode_hex &crc16 }
 
 sub crc16_base64 { _encode_base64 &crc16 }
 
+sub crcopenpgparmor_hex { _encode_hex &crcopenpgparmor }
+
+sub crcopenpgparmor_base64 { _encode_base64 &crcopenpgparmor }
+
 sub crc32_hex { _encode_hex &crc32 }
 
 sub crc32_base64 { _encode_base64 &crc32 }
@@ -320,15 +354,22 @@ Digest::CRC - Generic CRC functions
 
   # Functional style
 
-  use Digest::CRC qw(crc64 crc32 crc16 crcccitt crc crc8);
+  use Digest::CRC qw(crc64 crc32 crc16 crcccitt crc crc8 crcopenpgparmor);
   $crc = crc64("123456789");
   $crc = crc32("123456789");
   $crc = crc16("123456789");
   $crc = crcccitt("123456789");
   $crc = crc8("123456789");
+  $crc = crcopenpgparmor("123456789");
 
   $crc = crc($input,$width,$init,$xorout,$refout,$poly,$refin,$cont);
 
+
+  # add data to existing
+
+  $crc = crc32("ABCD", $crc);
+
+
   # OO style
   use Digest::CRC;
 
@@ -348,7 +389,12 @@ Digest::CRC - Generic CRC functions
 
 The B<Digest::CRC> module calculates CRC sums of all sorts.
 It contains wrapper functions with the correct parameters for CRC-CCITT,
-CRC-16, CRC-32 and CRC-64.
+CRC-16, CRC-32 and CRC-64, as well as the CRC used in OpenPGP's
+ASCII-armored checksum.
+
+=head1 SEE ALSO
+
+https://tools.ietf.org/html/rfc4880#section-6
 
 =head1 AUTHOR
 
